#!/usr/bin/python

################################################################
#                                                              #
# Blue Team Training Toolkit (BT3)                             #
# written by Juan J. Guelfo @ Encripto AS                      #
# post@encripto.no                                             #
#                                                              #
# Copyright 2013-2022 Encripto AS. All rights reserved.        #
#                                                              #
# BT3 is licensed under the FreeBSD license.                   #
# http://www.freebsd.org/copyright/freebsd-license.html        #
#                                                              #
################################################################


import cmd, os, signal, getpass, time, HTMLParser
import libs.bt3in, libs.bt3out, libs.bt3ver, libs.bt3api
import modules.maligno, modules.mocksum, modules.pcapteller


class Menu(cmd.Cmd):

    def __init__(self):
        cmd.Cmd.__init__(self)
        self.api_credentials = []
        self.hash_iterations = 50000
        self.maligno_menu = modules.maligno.Menu()
        self.mocksum_menu = modules.mocksum.Menu()
        self.pcapteller_menu = modules.pcapteller.Menu()


    def api_creds_preload(self, api_creds):
        self.api_credentials = api_creds
        self.maligno_menu.api_credentials = api_creds
        self.mocksum_menu.api_credentials = api_creds
        self.pcapteller_menu.api_credentials = api_creds


    def clear_api_creds(self):
        if len(self.api_credentials) == 2:
            self.api_credentials[0] = libs.bt3api.generate_string()
            self.api_credentials[1] = libs.bt3api.generate_string()
            self.api_credentials    = []

        if len(self.maligno_menu.api_credentials) == 2:
            self.maligno_menu.api_credentials[0] = libs.bt3api.generate_string()
            self.maligno_menu.api_credentials[1] = libs.bt3api.generate_string()
            self.maligno_menu.api_credentials    = []

        if len(self.mocksum_menu.api_credentials) == 2:
            self.mocksum_menu.api_credentials[0] = libs.bt3api.generate_string()
            self.mocksum_menu.api_credentials[1] = libs.bt3api.generate_string()
            self.mocksum_menu.api_credentials    = []

        if len(self.pcapteller_menu.api_credentials) == 2:
            self.pcapteller_menu.api_credentials[0] = libs.bt3api.generate_string()
            self.pcapteller_menu.api_credentials[1] = libs.bt3api.generate_string()
            self.pcapteller_menu.api_credentials    = []


    def complete_show(self, text, line, begidx, endidx):
        return [i for i in ["modules", "subscription"] if i.startswith(text)]


    def complete_use(self, text, line, begidx, endidx):
        return [i for i in ["maligno", "mocksum", "pcapteller"] if i.startswith(text)]


    def default(self, args):
        print("")
        libs.bt3out.print_error("%s\n" % libs.bt3out.INVALID_COMMAND)


    def do_apiconnect(self, args):
        print("")
        libs.bt3out.print_error("%s\n" % libs.bt3out.OFFLINE_MODE)


    def do_apidisconnect(self, args):
        print("")
        libs.bt3out.print_info("%s\n" % libs.bt3out.API_DISCONNECTED)


    def do_apinewcreds(self, args):
        print("")
        libs.bt3out.print_error("%s\n" % libs.bt3out.OFFLINE_MODE)


    def do_apiredeem(self, args):
        print("")
        libs.bt3out.print_error("%s\n" % libs.bt3out.OFFLINE_MODE)


    def do_apisignup(self, args):
        print("")
        libs.bt3out.print_error("%s\n" % libs.bt3out.OFFLINE_MODE)


    def do_apidelete(self, args):
        print("")
        libs.bt3out.print_error("%s\n" % libs.bt3out.OFFLINE_MODE)


    def do_back(self, args):
        pass


    def do_bt3update(self, args):
        print("")
        libs.bt3out.print_error("%s\n" % libs.bt3out.OFFLINE_MODE)


    def do_exit(self, args):
        self.clear_api_creds()
        libs.bt3out.exit_program(args)


    def do_help(self, args):
        cmds = [["apiconnect", "Connect to Blue Team Training Toolkit API with valid credentials."],
                ["apidelete", "Delete your Blue Team Training Toolkit content subscription account."],
                ["apidisconnect", "Disconnect from Blue Team Training Toolkit API, and work in offline mode."],
                ["apinewcreds", "Start a Blue Team Training Toolkit API password change or account recovery."],
                ["apiredeem <code>", "Redeem a Blue Team Training Toolkit credit voucher."],
                ["apisignup", "Create a new Blue Team Training Toolkit content subscription account."],
                ["back", "Exit current selected module and return to main menu."],
                ["bt3update", "Check for software updates."],
                ["exit", "Exit the Blue Team Training Toolkit."],
                ["help", "Display help menu."],
                ["resource <file>", "Run a sequence of Blue Team Training Toolkit commands from a resource file."],
                ["show modules", "Display supported application modules."],
                ["show subscription", "Display Blue Team Training Toolkit content subscription details."],
                ["use <module>", "Select an application module."],
                ["version", "Display software version."]]
        libs.bt3out.print_help(cmds, False, True)


    def do_resource(self, args):
        args = args.split(" ")
        if len(args) == 1 and args[0] not in [""]:
            try:
                if libs.bt3in.check_file_exists(args[0]):
                    cmd_list = []
                    cmd_list_menu = []
                    print("")
                    libs.bt3out.print_info("Running resource file...")
                    res_file = open(args[0], "r")
                    for line in res_file:
                        line = line.replace("\n", "")
                        if len(line) > 0:
                            cmd_list.append(line)

                    res_file.close()

                    module_reached = False
                    for cmd in cmd_list:
                        if not module_reached:
                            cmd_list_menu.append(cmd)
                            if cmd.upper() in ["USE MALIGNO", "USE MOCKSUM", "USE PCAPTELLER"]:
                                module_reached = True

                        else:
                            if cmd_list_menu[-1].upper() in ["USE MALIGNO"]:
                                self.maligno_menu.resource_cmds.append(cmd)
                                if cmd.upper() in ["BACK"]:
                                    libs.bt3out.print_warning("%s" % libs.bt3out.LIMIT_RESOURCE_FILE)
                                    libs.bt3out.print_warning("%s" % libs.bt3out.IGNORE_RESOURCE_FILE)
                                    break

                            elif cmd_list_menu[-1].upper() in ["USE MOCKSUM"]:
                                self.mocksum_menu.resource_cmds.append(cmd)
                                if cmd.upper() in ["BACK"]:
                                    libs.bt3out.print_warning("%s" % libs.bt3out.LIMIT_RESOURCE_FILE)
                                    libs.bt3out.print_warning("%s" % libs.bt3out.IGNORE_RESOURCE_FILE)
                                    break

                            elif cmd_list_menu[-1].upper() in ["USE PCAPTELLER"]:
                                self.pcapteller_menu.resource_cmds.append(cmd)
                                if cmd.upper() in ["BACK"]:
                                    libs.bt3out.print_warning("%s" % libs.bt3out.LIMIT_RESOURCE_FILE)
                                    libs.bt3out.print_warning("%s" % libs.bt3out.IGNORE_RESOURCE_FILE)
                                    break
                    print("")
                    for cmd in cmd_list_menu:
                        c = self.precmd(cmd)
                        e = self.onecmd(c)

                else:
                    print("")
                    libs.bt3out.print_error("%s\n" % libs.bt3out.INVALID_RESOURCE_FILE)

            except Exception as e:
                print("")
                libs.bt3out.print_error("%s\n" % e)

        else:
            cmds = [["resource <file>", "Run a sequence of BT3 commands from a resource file."]]
            libs.bt3out.print_help(cmds, True, True)


    def do_show(self, args):
        if args.upper() in ["MODULES"]:
            cmds = [["maligno", "Attack simulation with customized malware indicators."],
                    ["mocksum", "Repository of harmless files mimicking malware samples via hash collisions."],
                    ["pcapteller", "Network traffic manipulation and replay."]]
            libs.bt3out.print_help(cmds, False, False)

        elif args.upper() in ["SUBSCRIPTION"]:
            if len(self.api_credentials) == 2:
                try:
                    json_results = libs.bt3api.get_subscription(self.api_credentials[0], self.api_credentials[1])
                    if json_results and libs.bt3api.validate_json(json_results):
                        parsed_results = libs.bt3api.parse_json(json_results)
                        if parsed_results[-1]["Result"]:
                            del parsed_results[-1]
                            for subscription in parsed_results:
                                libs.bt3out.print_subscription_details(subscription)

                        else:
                            print("")
                            libs.bt3out.print_error("%s\n" % parsed_results[-1]["Msg"])

                    else:
                        print("")
                        libs.bt3out.print_error("%s" % libs.bt3out.API_INCOMMUNICATION)
                        libs.bt3out.print_error("%s\n" % libs.bt3out.CONTACT_SUPPORT)

                except Exception as e:
                    print("")
                    libs.bt3out.print_error("%s\n" % e)

            else:
                print("")
                libs.bt3out.print_error("%s\n" % libs.bt3out.API_AUTHENTICATION)

        else:
            cmds = [["show modules", "Display supported application modules."],
                    ["show subscription", "Display BT3 content subscription details."]]
            libs.bt3out.print_help(cmds, True, True)


    def do_use(self, args):
        if args.upper() in ["MALIGNO"]:
            self.maligno_menu.prompt = libs.bt3out.print_prompt("maligno")
            self.maligno_menu.cmdloop()

        elif args.upper() in ["MOCKSUM"]:
            self.mocksum_menu.prompt = libs.bt3out.print_prompt("mocksum")
            self.mocksum_menu.cmdloop()

        elif args.upper() in ["PCAPTELLER"]:
            self.pcapteller_menu.prompt = libs.bt3out.print_prompt("pcapteller")
            self.pcapteller_menu.cmdloop()

        else:
            cmds = [["use maligno", "Use maligno module."],
                    ["use mocksum", "Use mocksum module."],
                    ["use pcapteller", "Use pcapteller module."]]
            libs.bt3out.print_help(cmds, True, True)


    def do_version(self, args):
        print("")
        libs.bt3out.print_info("You are running %s version %s\n" % ("Blue Team Training Toolkit (BT3)", libs.bt3ver.__version__))


    def emptyline(self):
        pass





